@@ -1,7 +1,6 @@
 ;;; calc-alg.el --- algebraic functions for Calc
 
-;; Copyright (C) 1990, 1991, 1992, 1993, 2001, 2002, 2003, 2004,
-;;   2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
+;; Copyright (C) 1990-1993, 2001-2015 Free Software Foundation, Inc.
 
 ;; Author: David Gillespie <daveg@synaptics.com>
 ;; Maintainer: Jay Belanger  <jay.p.belanger@gmail.com>
@@ -357,6 +356,8 @@
 ;; math-simplify-step, which is called by math-simplify.
 (defvar math-top-only)
 
+;; math-normalize-error is declared in calc.el.
+(defvar math-normalize-error)
 (defun math-simplify (top-expr)
   (let ((math-simplifying t)
 	(math-top-only (consp calc-simplify-mode))
@@ -384,10 +385,12 @@
       (calc-with-default-simplification
        (while (let ((r simp-rules))
 		(setq res (math-normalize top-expr))
-		(while r
-		  (setq res (math-rewrite res (car r))
-			r (cdr r)))
-		(not (equal top-expr (setq res (math-simplify-step res)))))
+                (if (not math-normalize-error)
+                    (progn
+                      (while r
+                        (setq res (math-rewrite res (car r))
+                              r (cdr r)))
+                      (not (equal top-expr (setq res (math-simplify-step res)))))))
 	 (setq top-expr res)))))
   top-expr)
 
@@ -416,17 +419,14 @@
 
 
 (defmacro math-defsimplify (funcs &rest code)
-  (append '(progn)
-          (mapcar (function
-                   (lambda (func)
-                     (list 'put (list 'quote func) ''math-simplify
-                           (list 'nconc
-                                 (list 'get (list 'quote func) ''math-simplify)
-                                 (list 'list
-                                       (list 'function
-                                             (append '(lambda (math-simplify-expr))
-                                                     code)))))))
-                  (if (symbolp funcs) (list funcs) funcs))))
+  (cons 'progn
+        (mapcar #'(lambda (func)
+                    `(put ',func 'math-simplify
+                          (nconc
+                           (get ',func 'math-simplify)
+                           (list
+                            #'(lambda (math-simplify-expr) ,@code)))))
+                (if (symbolp funcs) (list funcs) funcs))))
 (put 'math-defsimplify 'lisp-indent-hook 1)
 
 ;; The function created by math-defsimplify uses the variable
@@ -534,7 +534,10 @@
                            (not (Math-realp (nth 1 math-simplify-expr))))
 		       (math-common-constant-factor (nth 1 math-simplify-expr))))
 	  (if (and (eq (car-safe nn) 'frac) (eq (nth 1 nn) 1) (not n))
-	      (progn
+	      (unless (and (eq (car-safe math-simplify-expr) 'calcFunc-eq)
+                           (eq (car-safe (nth 1 math-simplify-expr)) 'var)
+                           (not (math-expr-contains (nth 2 math-simplify-expr) 
+                                                    (nth 1 math-simplify-expr))))
 		(setcar (cdr math-simplify-expr)
                         (math-mul (nth 2 nn) (nth 1 math-simplify-expr)))
 		(setcar (cdr (cdr math-simplify-expr))
@@ -1659,11 +1662,11 @@
 ;; math-is-poly-rec.
 (defvar math-is-poly-degree)
 (defvar math-is-poly-loose)
-(defvar var)
+(defvar math-var)
 
-(defun math-is-polynomial (expr var &optional math-is-poly-degree math-is-poly-loose)
+(defun math-is-polynomial (expr math-var &optional math-is-poly-degree math-is-poly-loose)
   (let* ((math-poly-base-variable (if math-is-poly-loose
-				      (if (eq math-is-poly-loose 'gen) var '(var XXX XXX))
+				      (if (eq math-is-poly-loose 'gen) math-var '(var XXX XXX))
 				    math-poly-base-variable))
 	 (poly (math-is-poly-rec expr math-poly-neg-powers)))
     (and (or (null math-is-poly-degree)
@@ -1672,11 +1675,11 @@
 
 (defun math-is-poly-rec (expr negpow)
   (math-poly-simplify
-   (or (cond ((or (equal expr var)
+   (or (cond ((or (equal expr math-var)
 		  (eq (car-safe expr) '^))
 	      (let ((pow 1)
 		    (expr expr))
-		(or (equal expr var)
+		(or (equal expr math-var)
 		    (setq pow (nth 2 expr)
 			  expr (nth 1 expr)))
 		(or (eq math-poly-mult-powers 1)
@@ -1690,7 +1693,7 @@
 					 (equal math-poly-mult-powers
 						(nth 1 m))
 				       (setq math-poly-mult-powers (nth 1 m)))
-				     (or (equal expr var)
+				     (or (equal expr math-var)
 					 (eq math-poly-mult-powers 1))
 				     (car m)))))
 		(if (consp pow)
@@ -1698,7 +1701,7 @@
 		      (setq pow (math-to-simple-fraction pow))
 		      (and (eq (car-safe pow) 'frac)
 			   math-poly-frac-powers
-			   (equal expr var)
+			   (equal expr math-var)
 			   (setq math-poly-frac-powers
 				 (calcFunc-lcm math-poly-frac-powers
 					       (nth 2 pow))))))
@@ -1706,10 +1709,10 @@
 		    (setq pow (math-mul pow math-poly-frac-powers)))
 		(if (integerp pow)
 		    (if (and (= pow 1)
-			     (equal expr var))
+			     (equal expr math-var))
 			(list 0 1)
 		      (if (natnump pow)
-			  (let ((p1 (if (equal expr var)
+			  (let ((p1 (if (equal expr math-var)
 					(list 0 1)
 				      (math-is-poly-rec expr nil)))
 				(n pow)
@@ -1749,7 +1752,7 @@
                                     math-is-poly-degree))
 			    (math-poly-mul p1 p2))))))
 	     ((eq (car expr) '/)
-	      (and (or (not (math-poly-depends (nth 2 expr) var))
+	      (and (or (not (math-poly-depends (nth 2 expr) math-var))
 		       (and negpow
 			    (math-is-poly-rec (nth 2 expr) nil)
 			    (setq math-poly-neg-powers
@@ -1759,13 +1762,13 @@
 		     (mapcar (function (lambda (x) (math-div x (nth 2 expr))))
 			     p1))))
 	     ((and (eq (car expr) 'calcFunc-exp)
-		   (equal var '(var e var-e)))
-	      (math-is-poly-rec (list '^ var (nth 1 expr)) negpow))
+		   (equal math-var '(var e var-e)))
+	      (math-is-poly-rec (list '^ math-var (nth 1 expr)) negpow))
 	     ((and (eq (car expr) 'calcFunc-sqrt)
 		   math-poly-frac-powers)
 	      (math-is-poly-rec (list '^ (nth 1 expr) '(frac 1 2)) negpow))
 	     (t nil))
-       (and (or (not (math-poly-depends expr var))
+       (and (or (not (math-poly-depends expr math-var))
 		math-is-poly-loose)
 	    (not (eq (car expr) 'vec))
 	    (list expr)))))
@@ -1845,7 +1848,7 @@
 	     expr))))
 
 ;;; Simplify a polynomial in list form by stripping off high-end zeros.
-;;; This always leaves the constant part, i.e., nil->nil and nonnil->nonnil.
+;;; This always leaves the constant part, i.e., nil->nil and non-nil->non-nil.
 (defun math-poly-simplify (p)
   (and p
        (if (Math-zerop (nth (1- (length p)) p))
@@ -1914,5 +1917,4 @@
 
 (provide 'calc-alg)
 
-;; arch-tag: 52e7dcdf-9688-464d-a02b-4bbe789348d0
 ;;; calc-alg.el ends here
