@@ -1,6 +1,6 @@
 ;;; cal-china.el --- calendar functions for the Chinese calendar
 
-;; Copyright (C) 1995, 1997, 2001-2012  Free Software Foundation, Inc.
+;; Copyright (C) 1995, 1997, 2001-2015 Free Software Foundation, Inc.
 
 ;; Author: Edward M. Reingold <reingold@cs.uiuc.edu>
 ;; Maintainer: Glenn Morris <rgm@gnu.org>
@@ -327,19 +327,7 @@ Gregorian year Y-1 to the Chinese month of the solstice of Gregorian year Y."
 (defvar calendar-chinese-year-cache
   ;; Maintainers: delete existing value, position point at start of
   ;; empty line, then call  M-: (calendar-chinese-year-cache-init N)
-  '((2000 (12 730126) (1 730155) (2 730185) (3 730215) (4 730244) (5 730273)
-          (6 730303) (7 730332) (8 730361) (9 730391) (10 730420) (11 730450))
-    (2001 (12 730480) (1 730509) (2 730539) (3 730569) (4 730598) (4.5 730628)
-          (5 730657) (6 730687) (7 730716) (8 730745) (9 730775) (10 730804)
-          (11 730834))
-    (2002 (12 730863) (1 730893) (2 730923) (3 730953) (4 730982) (5 731012)
-          (6 731041) (7 731071) (8 731100) (9 731129) (10 731159) (11 731188))
-    (2003 (12 731218) (1 731247) (2 731277) (3 731307) (4 731336) (5 731366)
-          (6 731396) (7 731425) (8 731455) (9 731484) (10 731513) (11 731543))
-    (2004 (12 731572) (1 731602) (2 731631) (2.5 731661) (3 731690) (4 731720)
-          (5 731750) (6 731779) (7 731809) (8 731838) (9 731868) (10 731897)
-          (11 731927))
-    (2005 (12 731956) (1 731986) (2 732015) (3 732045) (4 732074) (5 732104)
+  '((2005 (12 731956) (1 731986) (2 732015) (3 732045) (4 732074) (5 732104)
           (6 732133) (7 732163) (8 732193) (9 732222) (10 732252) (11 732281))
     (2006 (12 732311) (1 732340) (2 732370) (3 732399) (4 732429) (5 732458)
           (6 732488) (7 732517) (7.5 732547) (8 732576) (9 732606) (10 732636)
@@ -376,7 +364,19 @@ Gregorian year Y-1 to the Chinese month of the solstice of Gregorian year Y."
           (6 737243) (7 737272) (8 737301) (9 737331) (10 737360) (11 737389))
     (2020 (12 737419) (1 737449) (2 737478) (3 737508) (4 737538) (4.5 737568)
           (5 737597) (6 737627) (7 737656) (8 737685) (9 737715) (10 737744)
-          (11 737774)))
+          (11 737774))
+    (2021 (12 737803) (1 737833) (2 737862) (3 737892) (4 737922) (5 737951)
+          (6 737981) (7 738010) (8 738040) (9 738069) (10 738099) (11 738128))
+    (2022 (12 738158) (1 738187) (2 738217) (3 738246) (4 738276) (5 738305)
+          (6 738335) (7 738365) (8 738394) (9 738424) (10 738453) (11 738483))
+    (2023 (12 738512) (1 738542) (2 738571) (2.5 738601) (3 738630) (4 738659)
+          (5 738689) (6 738719) (7 738748) (8 738778) (9 738808) (10 738837)
+          (11 738867))
+    (2024 (12 738896) (1 738926) (2 738955) (3 738985) (4 739014) (5 739043)
+          (6 739073) (7 739102) (8 739132) (9 739162) (10 739191) (11 739221))
+    (2025 (12 739251) (1 739280) (2 739310) (3 739339) (4 739369) (5 739398)
+          (6 739427) (6.5 739457) (7 739486) (8 739516) (9 739545) (10 739575)
+          (11 739605)))
   "Alist of Chinese year structures as determined by `chinese-year'.
 The default can be nil, but some values are precomputed for efficiency.")
 
