@@ -1,7 +1,6 @@
 ;;; semantic/java.el --- Semantic functions for Java
 
-;;; Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
-;;; 2007, 2008, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
+;;; Copyright (C) 1999-2015 Free Software Foundation, Inc.
 
 ;; Author: David Ponce <david@dponce.com>
 
@@ -122,6 +121,7 @@ corresponding compound declaration."
       (setq clone (semantic-tag-clone tag (car dim))
             xpand (cons clone xpand))
       (semantic-tag-put-attribute clone :dereference (cdr dim)))
+
      ((eq class 'variable)
       (or (consp elts) (setq elts (list (list elts))))
       (setq dim  (semantic-java-dim (semantic-tag-get-attribute tag :type))
@@ -140,7 +140,20 @@ corresponding compound declaration."
         (semantic-tag-put-attribute clone :type type)
         (semantic-tag-put-attribute clone :dereference (+ dim0 (cdr dim)))
         (semantic-tag-set-bounds clone start end)))
-     )
+
+     ((and (eq class 'type) (string-match "\\." (semantic-tag-name tag)))
+      ;; javap outputs files where the package name is stuck onto the class or interface
+      ;; name.  To make this more regular, we extract the package name into a package statement,
+      ;; then make the class name regular.
+      (let* ((name (semantic-tag-name tag))
+	     (rsplit (nreverse (split-string name "\\." t)))
+	     (newclassname (car rsplit))
+	     (newpkg (mapconcat 'identity (reverse (cdr rsplit)) ".")))
+	(semantic-tag-set-name tag newclassname)
+	(setq xpand
+	      (list tag
+		    (semantic-tag-new-package newpkg nil))))
+      ))
     xpand))
 
 ;;; Environment
@@ -160,6 +173,15 @@ corresponding compound declaration."
           (semantic-find-tags-by-class
            'type (semantic-find-tag-by-overlay point))))
 
+;; Tag Protection
+;;
+(define-mode-local-override semantic-tag-protection
+  java-mode (tag &optional parent)
+  "Return the protection of TAG in PARENT.
+Override function for `semantic-tag-protection'."
+  (let ((prot (semantic-tag-protection-default tag parent)))
+    (or prot 'package)))
+
 ;; Prototype handler
 ;;
 (defun semantic-java-prototype-function (tag &optional parent color)
@@ -243,7 +265,6 @@ Optional argument COLOR indicates that color should be mixed in."
   (let ((name (semantic-tag-name tag)))
     (concat (mapconcat 'identity (split-string name "\\.") "/") ".java")))
 
-
 ;; Documentation handler
 ;;
 (defsubst semantic-java-skip-spaces-backward ()
@@ -257,7 +278,7 @@ Optional argument COLOR indicates that color should be mixed in."
 (define-mode-local-override semantic-documentation-for-tag
   java-mode (&optional tag nosnarf)
   "Find documentation from TAG and return it as a clean string.
-Java has documentation set in a comment preceding TAG's definition.
+Java have documentation set in a comment preceding TAG's definition.
 Attempt to strip out comment syntactic sugar, unless optional argument
 NOSNARF is non-nil.
 If NOSNARF is 'lex, then return the semantic lex token."
@@ -459,5 +480,8 @@ removed from the result list."
 
 (provide 'semantic/java)
 
-;; arch-tag: ca14cb88-9343-48a9-9aa0-14d05be8d49f
+;; Local variables:
+;; generated-autoload-load-name: "semantic/java"
+;; End:
+
 ;;; semantic/java.el ends here
