@@ -1,6 +1,6 @@
 ;;; srecode/map.el --- Manage a template file map
 
-;; Copyright (C) 2008, 2009, 2010 Free Software Foundation, Inc.
+;; Copyright (C) 2008-2015 Free Software Foundation, Inc.
 
 ;; Author: Eric M. Ludlam <eric@siege-engine.com>
 
@@ -80,7 +80,7 @@ Each app keys to an alist of files and modes (as above.)")
     ans))
 
 (defmethod srecode-map-entry-for-app ((map srecode-map) app)
-  "Return the entry in MAP for APP'lication."
+  "Return the entry in MAP for APP."
   (assoc app (oref map apps))
   )
 
@@ -137,7 +137,7 @@ Return non-nil if the MAP was changed."
     dirty))
 
 (defmethod srecode-map-delete-file-entry-from-app ((map srecode-map) file app)
-  "Delete from MAP the FILE entry within the APP'lication."
+  "Delete from MAP the FILE entry within the APP."
   (let* ((appe (srecode-map-entry-for-app map app))
 	 (fentry (assoc file (cdr appe))))
     (setcdr appe (delete fentry (cdr appe))))
@@ -215,7 +215,8 @@ Optional argument RESET forces a reset of the current map."
     ;; Eventually, I want to return many maps to search through.
     (list srecode-current-map)))
 
-(eval-when-compile (require 'data-debug))
+(declare-function data-debug-new-buffer "data-debug")
+(declare-function data-debug-insert-stuff-list "data-debug")
 
 (defun srecode-adebug-maps ()
   "Run ADEBUG on the output of `srecode-get-maps'."
@@ -295,8 +296,14 @@ if that file is NEW, otherwise assume the mode has not changed."
 
     ;; 2) Do we not have a current map?  If so load.
     (when (not srecode-current-map)
-      (setq srecode-current-map
-	    (eieio-persistent-read srecode-map-save-file))
+      (condition-case nil
+	  (setq srecode-current-map
+		(eieio-persistent-read srecode-map-save-file srecode-map))
+	(error
+	 ;; There was an error loading the old map.  Create a new one.
+	 (setq srecode-current-map
+	       (srecode-map "SRecode Map"
+			    :file srecode-map-save-file))))
       )
 
     )
@@ -356,6 +363,9 @@ Return non-nil if the map changed."
 	  (let ((semantic-init-hook nil))
 	    (semantic-new-buffer-fcn))
 	  )
+	;; Force semantic to be enabled in this buffer.
+	(unless (semantic-active-p)
+	  (semantic-new-buffer-fcn))
 
 	(semantic-fetch-tags)
 	(let* ((mode-tag
@@ -409,5 +419,4 @@ Return non-nil if the map changed."
 ;; generated-autoload-load-name: "srecode/map"
 ;; End:
 
-;; arch-tag: dc90c737-1e87-455a-bbd1-6b72cdbfb7fd
 ;;; srecode/map.el ends here
