@@ -1,6 +1,6 @@
 ;;; edmacro.el --- keyboard macro editor
 
-;; Copyright (C) 1993, 1994 Free Software Foundation, Inc.
+;; Copyright (C) 1993-1994, 2001-2015 Free Software Foundation, Inc.
 
 ;; Author: Dave Gillespie <daveg@synaptics.com>
 ;; Maintainer: Dave Gillespie <daveg@synaptics.com>
@@ -9,10 +9,10 @@
 
 ;; This file is part of GNU Emacs.
 
-;; GNU Emacs is free software; you can redistribute it and/or modify
+;; GNU Emacs is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; the Free Software Foundation, either version 3 of the License, or
+;; (at your option) any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,25 +20,23 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-;; Boston, MA 02111-1307, USA.
+;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
 ;;; Usage:
 ;;
-;; The `C-x C-k' (`edit-kbd-macro') command edits a keyboard macro
+;; The `C-x C-k e' (`edit-kbd-macro') command edits a keyboard macro
 ;; in a special buffer.  It prompts you to type a key sequence,
 ;; which should be one of:
 ;;
-;;  * RET or `C-x e' (call-last-kbd-macro), to edit the most 
+;;  * RET or `C-x e' (call-last-kbd-macro), to edit the most
 ;;    recently defined keyboard macro.
 ;;
 ;;  * `M-x' followed by a command name, to edit a named command
 ;;    whose definition is a keyboard macro.
 ;;
-;;  * `C-h l' (view-lossage), to edit the 100 most recent keystrokes
+;;  * `C-h l' (view-lossage), to edit the 300 most recent keystrokes
 ;;    and install them as the "current" macro.
 ;;
 ;;  * any key sequence whose definition is a keyboard macro.
@@ -63,30 +61,24 @@
 ;; With a prefix argument, `edit-kbd-macro' will format the
 ;; macro in a more concise way that omits the comments.
 
-;; This package requires GNU Emacs 19 or later, and daveg's CL
-;; package 2.02 or later.  (CL 2.02 comes standard starting with
-;; Emacs 19.18.)  This package does not work with Emacs 18 or
-;; Lucid Emacs.
-
 ;;; Code:
-
-(eval-when-compile
- (require 'cl))
+
+(require 'cl-lib)
+(require 'kmacro)
 
 ;;; The user-level commands for editing macros.
 
-;;;###autoload (define-key ctl-x-map "\C-k" 'edit-kbd-macro)
+(defcustom edmacro-eight-bits nil
+  "Non-nil if `edit-kbd-macro' should leave 8-bit characters intact.
+Default nil means to write characters above \\177 in octal notation."
+  :type 'boolean
+  :group 'kmacro)
 
-;;;###autoload
-(defvar edmacro-eight-bits nil
-  "*Non-nil if edit-kbd-macro should leave 8-bit characters intact.
-Default nil means to write characters above \\177 in octal notation.")
-
-(defvar edmacro-mode-map nil)
-(unless edmacro-mode-map
-  (setq edmacro-mode-map (make-sparse-keymap))
-  (define-key edmacro-mode-map "\C-c\C-c" 'edmacro-finish-edit)
-  (define-key edmacro-mode-map "\C-c\C-q" 'edmacro-insert-key))
+(defvar edmacro-mode-map
+  (let ((map (make-sparse-keymap)))
+    (define-key map "\C-c\C-c" 'edmacro-finish-edit)
+    (define-key map "\C-c\C-q" 'edmacro-insert-key)
+    map))
 
 (defvar edmacro-store-hook)
 (defvar edmacro-finish-hook)
@@ -97,30 +89,35 @@ Default nil means to write characters above \\177 in octal notation.")
   "Edit a keyboard macro.
 At the prompt, type any key sequence which is bound to a keyboard macro.
 Or, type `C-x e' or RET to edit the last keyboard macro, `C-h l' to edit
-the last 100 keystrokes as a keyboard macro, or `M-x' to edit a macro by
+the last 300 keystrokes as a keyboard macro, or `M-x' to edit a macro by
 its command name.
 With a prefix argument, format the macro in a more concise way."
   (interactive "kKeyboard macro to edit (C-x e, M-x, C-h l, or keys): \nP")
   (when keys
     (let ((cmd (if (arrayp keys) (key-binding keys) keys))
-	  (mac nil))
+	  (mac nil) (mac-counter nil) (mac-format nil)
+	  kmacro)
       (cond (store-hook
 	     (setq mac keys)
 	     (setq cmd nil))
-	    ((or (eq cmd 'call-last-kbd-macro)
+	    ((or (memq cmd '(call-last-kbd-macro kmacro-call-macro
+			     kmacro-end-or-call-macro kmacro-end-and-call-macro))
 		 (member keys '("\r" [return])))
 	     (or last-kbd-macro
 		 (y-or-n-p "No keyboard macro defined.  Create one? ")
 		 (keyboard-quit))
 	     (setq mac (or last-kbd-macro ""))
+	     (setq keys nil)
 	     (setq cmd 'last-kbd-macro))
 	    ((eq cmd 'execute-extended-command)
 	     (setq cmd (read-command "Name of keyboard macro to edit: "))
 	     (if (string-equal cmd "")
 		 (error "No command name given"))
+	     (setq keys nil)
 	     (setq mac (symbol-function cmd)))
 	    ((memq cmd '(view-lossage electric-view-lossage))
 	     (setq mac (recent-keys))
+	     (setq keys nil)
 	     (setq cmd 'last-kbd-macro))
 	    ((null cmd)
 	     (error "Key sequence %s is not defined" (key-description keys)))
@@ -129,6 +126,10 @@ With a prefix argument, format the macro in a more concise way."
 	    (t
 	     (setq mac cmd)
 	     (setq cmd nil)))
+      (when (setq kmacro (kmacro-extract-lambda mac))
+	(setq mac (car kmacro)
+	      mac-counter (nth 1 kmacro)
+	      mac-format (nth 2 kmacro)))
       (unless (arrayp mac)
 	(error "Key sequence %s is not a keyboard macro"
 	       (key-description keys)))
@@ -154,11 +155,15 @@ With a prefix argument, format the macro in a more concise way."
 	(insert ";; Original keys: " fmt "\n")
 	(unless store-hook
 	  (insert "\nCommand: " (if cmd (symbol-name cmd) "none") "\n")
-	  (let ((keys (where-is-internal (or cmd mac) '(keymap))))
-	    (if keys
-		(while keys
-		  (insert "Key: " (edmacro-format-keys (pop keys) 1) "\n"))
-	      (insert "Key: none\n"))))
+	  (let ((gkeys (where-is-internal (or cmd mac) '(keymap))))
+	    (if (and keys (not (member keys gkeys)))
+		(setq gkeys (cons keys gkeys)))
+	    (if gkeys
+		(while gkeys
+		  (insert "Key: " (edmacro-format-keys (pop gkeys) 1) "\n"))
+	      (insert "Key: none\n")))
+	  (when (and mac-counter mac-format)
+	    (insert (format "Counter: %d\nFormat: \"%s\"\n" mac-counter mac-format))))
 	(insert "\nMacro:\n\n")
 	(save-excursion
 	  (insert fmtv "\n"))
@@ -218,6 +223,7 @@ or nil, use a compact 80-column format."
      "This command is valid only in buffers created by `edit-kbd-macro'"))
   (run-hooks 'edmacro-finish-hook)
   (let ((cmd nil) (keys nil) (no-keys nil)
+	(mac-counter nil) (mac-format nil)
 	(top (point-min)))
     (goto-char top)
     (let ((case-fold-search nil))
@@ -232,6 +238,7 @@ or nil, use a compact 80-column format."
 			(setq cmd (and (not (equal str "none"))
 				       (intern str)))
 			(and (fboundp cmd) (not (arrayp (symbol-function cmd)))
+			     (not (get cmd 'kmacro))
 			     (not (y-or-n-p
 				   (format "Command %s is already defined; %s"
 					   cmd "proceed? ")))
@@ -249,14 +256,32 @@ or nil, use a compact 80-column format."
 			  (push key keys)
 			  (let ((b (key-binding key)))
 			    (and b (commandp b) (not (arrayp b))
+				 (not (kmacro-extract-lambda b))
 				 (or (not (fboundp b))
-				     (not (arrayp (symbol-function b))))
+				     (not (or (arrayp (symbol-function b))
+					      (get b 'kmacro))))
 				 (not (y-or-n-p
 				       (format "Key %s is already defined; %s"
 					       (edmacro-format-keys key 1)
 					       "proceed? ")))
 				 (keyboard-quit))))))
 		    t)
+		   ((looking-at "Counter:[ \t]*\\([^ \t\n]*\\)[ \t]*$")
+		    (when edmacro-store-hook
+		      (error "\"Counter\" line not allowed in this context"))
+		    (let ((str (buffer-substring (match-beginning 1)
+						 (match-end 1))))
+		      (unless (equal str "")
+			(setq mac-counter (string-to-number str))))
+		    t)
+		   ((looking-at "Format:[ \t]*\"\\([^\n]*\\)\"[ \t]*$")
+		    (when edmacro-store-hook
+		      (error "\"Format\" line not allowed in this context"))
+		    (let ((str (buffer-substring (match-beginning 1)
+						 (match-end 1))))
+		      (unless (equal str "")
+			(setq mac-format str)))
+		    t)
 		   ((looking-at "Macro:[ \t\n]*")
 		    (goto-char (match-end 0))
 		    nil)
@@ -286,16 +311,24 @@ or nil, use a compact 80-column format."
 	    (when cmd
 	      (if (= (length mac) 0)
 		  (fmakunbound cmd)
-		(fset cmd mac)))
+		(fset cmd
+		      (if (and mac-counter mac-format)
+			  (kmacro-lambda-form mac mac-counter mac-format)
+			mac))))
 	    (if no-keys
 		(when cmd
-		  (loop for key in (where-is-internal cmd '(keymap)) do
-			(global-unset-key key)))
+		  (cl-loop for key in (where-is-internal cmd '(keymap)) do
+                           (global-unset-key key)))
 	      (when keys
 		(if (= (length mac) 0)
-		    (loop for key in keys do (global-unset-key key))
-		  (loop for key in keys do
-			(global-set-key key (or cmd mac)))))))))
+		    (cl-loop for key in keys do (global-unset-key key))
+		  (cl-loop for key in keys do
+                           (global-set-key key
+                                           (or cmd
+                                               (if (and mac-counter mac-format)
+                                                   (kmacro-lambda-form
+                                                    mac mac-counter mac-format)
+                                                 mac))))))))))
       (kill-buffer buf)
       (when (buffer-name obuf)
 	(switch-to-buffer obuf))
@@ -310,7 +343,7 @@ or nil, use a compact 80-column format."
     (insert (edmacro-format-keys key) " ")))
 
 (defun edmacro-mode ()
-  "\\<edmacro-mode-map>Keyboard Macro Editing mode.  Press
+  "\\<edmacro-mode-map>Keyboard Macro Editing mode.  Press \
 \\[edmacro-finish-edit] to save and exit.
 To abort the edit, just kill this buffer with \\[kill-buffer] RET.
 
@@ -389,10 +422,7 @@ doubt, use whitespace."
 
 (defun edmacro-format-keys (macro &optional verbose)
   (setq macro (edmacro-fix-menu-commands macro))
-  (let* ((maps (append (current-minor-mode-maps)
-		       (if (current-local-map)
-			   (list (current-local-map)))
-		       (list (current-global-map))))
+  (let* ((maps (current-active-maps))
 	 (pkeys '(end-macro ?0 ?1 ?2 ?3 ?4 ?5 ?6 ?7 ?8 ?9 ?- ?\C-u
 		  ?\M-- ?\M-0 ?\M-1 ?\M-2 ?\M-3 ?\M-4 ?\M-5 ?\M-6
 		  ?\M-7 ?\M-8 ?\M-9))
@@ -406,68 +436,69 @@ doubt, use whitespace."
 	 (one-line (eq verbose 1)))
     (if one-line (setq verbose nil))
     (when (stringp macro)
-      (loop for i below (length macro) do
-	    (when (>= (aref rest-mac i) 128)
-	      (incf (aref rest-mac i) (- ?\M-\^@ 128)))))
+      (cl-loop for i below (length macro) do
+               (when (>= (aref rest-mac i) 128)
+                 (cl-incf (aref rest-mac i) (- ?\M-\^@ 128)))))
     (while (not (eq (aref rest-mac 0) 'end-macro))
       (let* ((prefix
 	      (or (and (integerp (aref rest-mac 0))
 		       (memq (aref rest-mac 0) mdigs)
-		       (memq (key-binding (edmacro-subseq rest-mac 0 1))
+		       (memq (key-binding (cl-subseq rest-mac 0 1))
 			     '(digit-argument negative-argument))
 		       (let ((i 1))
 			 (while (memq (aref rest-mac i) (cdr mdigs))
-			   (incf i))
+			   (cl-incf i))
 			 (and (not (memq (aref rest-mac i) pkeys))
-			      (prog1 (concat "M-" (edmacro-subseq rest-mac 0 i) " ")
-				(callf edmacro-subseq rest-mac i)))))
+			      (prog1 (vconcat "M-" (cl-subseq rest-mac 0 i) " ")
+				(cl-callf cl-subseq rest-mac i)))))
 		  (and (eq (aref rest-mac 0) ?\C-u)
 		       (eq (key-binding [?\C-u]) 'universal-argument)
 		       (let ((i 1))
 			 (while (eq (aref rest-mac i) ?\C-u)
-			   (incf i))
+			   (cl-incf i))
 			 (and (not (memq (aref rest-mac i) pkeys))
-			      (prog1 (loop repeat i concat "C-u ")
-				(callf edmacro-subseq rest-mac i)))))
+			      (prog1 (cl-loop repeat i concat "C-u ")
+				(cl-callf cl-subseq rest-mac i)))))
 		  (and (eq (aref rest-mac 0) ?\C-u)
 		       (eq (key-binding [?\C-u]) 'universal-argument)
 		       (let ((i 1))
 			 (when (eq (aref rest-mac i) ?-)
-			   (incf i))
+			   (cl-incf i))
 			 (while (memq (aref rest-mac i)
 				      '(?0 ?1 ?2 ?3 ?4 ?5 ?6 ?7 ?8 ?9))
-			   (incf i))
+			   (cl-incf i))
 			 (and (not (memq (aref rest-mac i) pkeys))
-			      (prog1 (concat "C-u " (edmacro-subseq rest-mac 1 i) " ")
-				(callf edmacro-subseq rest-mac i)))))))
+			      (prog1 (vconcat "C-u " (cl-subseq rest-mac 1 i) " ")
+				(cl-callf cl-subseq rest-mac i)))))))
 	     (bind-len (apply 'max 1
-			      (loop for map in maps
-				    for b = (lookup-key map rest-mac)
-				    when b collect b)))
-	     (key (edmacro-subseq rest-mac 0 bind-len))
+			      (cl-loop for map in maps
+                                       for b = (lookup-key map rest-mac)
+                                       when b collect b)))
+	     (key (cl-subseq rest-mac 0 bind-len))
 	     (fkey nil) tlen tkey
-	     (bind (or (loop for map in maps for b = (lookup-key map key)
-			     thereis (and (not (integerp b)) b))
-		       (and (setq fkey (lookup-key function-key-map rest-mac))
-			    (setq tlen fkey tkey (edmacro-subseq rest-mac 0 tlen)
-				  fkey (lookup-key function-key-map tkey))
-			    (loop for map in maps
-				  for b = (lookup-key map fkey)
-				  when (and (not (integerp b)) b)
-				  do (setq bind-len tlen key tkey)
-				  and return b
-				  finally do (setq fkey nil)))))
+	     (bind (or (cl-loop for map in maps for b = (lookup-key map key)
+                                thereis (and (not (integerp b)) b))
+		       (and (setq fkey (lookup-key local-function-key-map rest-mac))
+			    (setq tlen fkey tkey (cl-subseq rest-mac 0 tlen)
+				  fkey (lookup-key local-function-key-map tkey))
+			    (cl-loop for map in maps
+                                     for b = (lookup-key map fkey)
+                                     when (and (not (integerp b)) b)
+                                     do (setq bind-len tlen key tkey)
+                                     and return b
+                                     finally do (setq fkey nil)))))
 	     (first (aref key 0))
-	     (text (loop for i from bind-len below (length rest-mac)
-			 for ch = (aref rest-mac i)
-			 while (and (integerp ch)
-				    (> ch 32) (< ch maxkey) (/= ch 92)
-				    (eq (key-binding (char-to-string ch))
-					'self-insert-command)
-				    (or (> i (- (length rest-mac) 2))
-					(not (eq ch (aref rest-mac (+ i 1))))
-					(not (eq ch (aref rest-mac (+ i 2))))))
-			 finally return i))
+	     (text
+              (cl-loop for i from bind-len below (length rest-mac)
+                       for ch = (aref rest-mac i)
+                       while (and (integerp ch)
+                                  (> ch 32) (< ch maxkey) (/= ch 92)
+                                  (eq (key-binding (char-to-string ch))
+                                      'self-insert-command)
+                                  (or (> i (- (length rest-mac) 2))
+                                      (not (eq ch (aref rest-mac (+ i 1))))
+                                      (not (eq ch (aref rest-mac (+ i 2))))))
+                       finally return i))
 	     desc)
 	(if (stringp bind) (setq bind nil))
 	(cond ((and (eq bind 'self-insert-command) (not prefix)
@@ -475,10 +506,10 @@ doubt, use whitespace."
 		    (> first 32) (<= first maxkey) (/= first 92)
 		    (progn
 		      (if (> text 30) (setq text 30))
-		      (setq desc (concat (edmacro-subseq rest-mac 0 text)))
+		      (setq desc (concat (cl-subseq rest-mac 0 text)))
 		      (when (string-match "^[ACHMsS]-." desc)
 			(setq text 2)
-			(callf substring desc 0 2))
+			(cl-callf substring desc 0 2))
 		      (not (string-match
 			    "^;;\\|^<.*>$\\|^\\\\[0-9]+$\\|^[0-9]+\\*."
 			    desc))))
@@ -492,7 +523,7 @@ doubt, use whitespace."
 		    (> text bind-len)
 		    (memq (aref rest-mac text) '(return 13))
 		    (progn
-		      (setq desc (concat (edmacro-subseq rest-mac bind-len text)))
+		      (setq desc (concat (cl-subseq rest-mac bind-len text)))
 		      (commandp (intern-soft desc))))
 	       (if (commandp (intern-soft desc)) (setq bind desc))
 	       (setq desc (format "<<%s>>" desc))
@@ -504,17 +535,17 @@ doubt, use whitespace."
 			      (cond
 			       ((integerp ch)
 				(concat
-				 (loop for pf across "ACHMsS"
-				       for bit in '(?\A-\^@ ?\C-\^@ ?\H-\^@
-						    ?\M-\^@ ?\s-\^@ ?\S-\^@)
-				       when (/= (logand ch bit) 0)
-				       concat (format "%c-" pf))
+				 (cl-loop for pf across "ACHMsS"
+                                          for bit in '(?\A-\^@ ?\C-\^@ ?\H-\^@
+                                                       ?\M-\^@ ?\s-\^@ ?\S-\^@)
+                                          when (/= (logand ch bit) 0)
+                                          concat (format "%c-" pf))
 				 (let ((ch2 (logand ch (1- (lsh 1 18)))))
 				   (cond ((<= ch2 32)
-					  (case ch2
+					  (pcase ch2
 					    (0 "NUL") (9 "TAB") (10 "LFD")
 					    (13 "RET") (27 "ESC") (32 "SPC")
-					    (t
+					    (_
 					     (format "C-%c"
 						     (+ (if (<= ch2 26) 96 64)
 							ch2)))))
@@ -526,121 +557,97 @@ doubt, use whitespace."
 			       (t
 				(error "Unrecognized item in macro: %s" ch)))))
 			   (or fkey key) " "))))
-	(if prefix (setq desc (concat prefix desc)))
+	(if prefix
+	    (setq desc (concat (edmacro-sanitize-for-string prefix) desc)))
 	(unless (string-match " " desc)
 	  (let ((times 1) (pos bind-len))
-	    (while (not (edmacro-mismatch rest-mac rest-mac
-					  0 bind-len pos (+ bind-len pos)))
-	      (incf times)
-	      (incf pos bind-len))
+	    (while (not (cl-mismatch rest-mac rest-mac
+				     :start1 0 :end1 bind-len
+				     :start2 pos :end2 (+ bind-len pos)))
+	      (cl-incf times)
+	      (cl-incf pos bind-len))
 	    (when (> times 1)
 	      (setq desc (format "%d*%s" times desc))
 	      (setq bind-len (* bind-len times)))))
-	(setq rest-mac (edmacro-subseq rest-mac bind-len))
+	(setq rest-mac (cl-subseq rest-mac bind-len))
 	(if verbose
 	    (progn
-	      (unless (equal res "") (callf concat res "\n"))
-	      (callf concat res desc)
+	      (unless (equal res "") (cl-callf concat res "\n"))
+	      (cl-callf concat res desc)
 	      (when (and bind (or (stringp bind) (symbolp bind)))
-		(callf concat res
+		(cl-callf concat res
 		  (make-string (max (- 3 (/ (length desc) 8)) 1) 9)
 		  ";; " (if (stringp bind) bind (symbol-name bind))))
 	      (setq len 0))
 	  (if (and (> (+ len (length desc) 2) 72) (not one-line))
 	      (progn
-		(callf concat res "\n ")
+		(cl-callf concat res "\n ")
 		(setq len 1))
 	    (unless (equal res "")
-	      (callf concat res " ")
-	      (incf len)))
-	  (callf concat res desc)
-	  (incf len (length desc)))))
+	      (cl-callf concat res " ")
+	      (cl-incf len)))
+	  (cl-callf concat res desc)
+	  (cl-incf len (length desc)))))
     res))
 
-(defun edmacro-mismatch (cl-seq1 cl-seq2 cl-start1 cl-end1 cl-start2 cl-end2)
-  "Compare SEQ1 with SEQ2, return index of first mismatching element.
-Return nil if the sequences match.  If one sequence is a prefix of the
-other, the return value indicates the end of the shorted sequence."
-  (let (cl-test cl-test-not cl-key cl-from-end)
-    (or cl-end1 (setq cl-end1 (length cl-seq1)))
-    (or cl-end2 (setq cl-end2 (length cl-seq2)))
-    (if cl-from-end
-	(progn
-	  (while (and (< cl-start1 cl-end1) (< cl-start2 cl-end2)
-		      (cl-check-match (elt cl-seq1 (1- cl-end1))
-				      (elt cl-seq2 (1- cl-end2))))
-	    (setq cl-end1 (1- cl-end1) cl-end2 (1- cl-end2)))
-	  (and (or (< cl-start1 cl-end1) (< cl-start2 cl-end2))
-	       (1- cl-end1)))
-      (let ((cl-p1 (and (listp cl-seq1) (nthcdr cl-start1 cl-seq1)))
-	    (cl-p2 (and (listp cl-seq2) (nthcdr cl-start2 cl-seq2))))
-	(while (and (< cl-start1 cl-end1) (< cl-start2 cl-end2)
-		    (cl-check-match (if cl-p1 (car cl-p1)
-				      (aref cl-seq1 cl-start1))
-				    (if cl-p2 (car cl-p2)
-				      (aref cl-seq2 cl-start2))))
-	  (setq cl-p1 (cdr cl-p1) cl-p2 (cdr cl-p2)
-		cl-start1 (1+ cl-start1) cl-start2 (1+ cl-start2)))
-	(and (or (< cl-start1 cl-end1) (< cl-start2 cl-end2))
-	     cl-start1)))))
+(defun edmacro-sanitize-for-string (seq)
+  "Convert a key sequence vector SEQ into a string.
+The string represents the same events; Meta is indicated by bit 7.
+This function assumes that the events can be stored in a string."
+  (setq seq (copy-sequence seq))
+  (cl-loop for i below (length seq) do
+           (when (logand (aref seq i) 128)
+             (setf (aref seq i) (logand (aref seq i) 127))))
+  seq)
 
-(defun edmacro-subseq (seq start &optional end)
-  "Return the subsequence of SEQ from START to END.
-If END is omitted, it defaults to the length of the sequence.
-If START or END is negative, it counts from the end."
-  (if (stringp seq) (substring seq start end)
-    (let (len)
-      (and end (< end 0) (setq end (+ end (setq len (length seq)))))
-      (if (< start 0) (setq start (+ start (or len (setq len (length seq))))))
-      (cond ((listp seq)
-	     (if (> start 0) (setq seq (nthcdr start seq)))
-	     (if end
-		 (let ((res nil))
-		   (while (>= (setq end (1- end)) start)
-		     (cl-push (cl-pop seq) res))
-		   (nreverse res))
-	       (copy-sequence seq)))
-	    (t
-	     (or end (setq end (or len (length seq))))
-	     (let ((res (make-vector (max (- end start) 0) nil))
-		   (i 0))
-	       (while (< start end)
-		 (aset res i (aref seq start))
-		 (setq i (1+ i) start (1+ start)))
-	       res))))))
-
-(defun edmacro-fix-menu-commands (macro)
-  (when (vectorp macro)
-    (let ((i 0) ev)
-      (while (< i (length macro))
-	(when (consp (setq ev (aref macro i)))
-	  (cond ((equal (cadadr ev) '(menu-bar))
-		 (setq macro (vconcat (edmacro-subseq macro 0 i)
-				      (vector 'menu-bar (car ev))
-				      (edmacro-subseq macro (1+ i))))
-		 (incf i))
+(defun edmacro-fix-menu-commands (macro &optional noerror)
+  (if (vectorp macro)
+      (let (result)
+	;; Make a list of the elements.
+	(setq macro (append macro nil))
+	(dolist (ev macro)
+	  (cond ((atom ev)
+		 (push ev result))
+		((eq (car ev) 'help-echo))
+		((eq (car ev) 'switch-frame))
+		((equal ev '(menu-bar))
+		 (push 'menu-bar result))
+		((equal (cl-cadadr ev) '(menu-bar))
+		 (push (vector 'menu-bar (car ev)) result))
 		;; It would be nice to do pop-up menus, too, but not enough
 		;; info is recorded in macros to make this possible.
+		(noerror
+		 ;; Just ignore mouse events.
+		 nil)
 		(t
 		 (error "Macros with mouse clicks are not %s"
 			"supported by this command"))))
-	(incf i))))
-  macro)
+	;; Reverse them again and make them back into a vector.
+	(vconcat (nreverse result)))
+    macro))
 
 ;;; Parsing a human-readable keyboard macro.
 
 (defun edmacro-parse-keys (string &optional need-vector)
   (let ((case-fold-search nil)
+	(len (length string)) ; We won't alter string in the loop below.
 	(pos 0)
 	(res []))
-    (while (and (< pos (length string))
+    (while (and (< pos len)
 		(string-match "[^ \t\n\f]+" string pos))
-      (let ((word (substring string (match-beginning 0) (match-end 0)))
-	    (key nil)
-	    (times 1))
-	(setq pos (match-end 0))
+      (let* ((word-beg (match-beginning 0))
+	     (word-end (match-end 0))
+	     (word (substring string word-beg len))
+	     (times 1)
+	     key)
+	;; Try to catch events of the form "<as df>".
+	(if (string-match "\\`<[^ <>\t\n\f][^>\t\n\f]*>" word)
+	    (setq word (match-string 0 word)
+		  pos (+ word-beg (match-end 0)))
+	  (setq word (substring string word-beg word-end)
+		pos word-end))
 	(when (string-match "\\([0-9]+\\)\\*." word)
-	  (setq times (string-to-int (substring word 0 (match-end 1))))
+	  (setq times (string-to-number (substring word 0 (match-end 1))))
 	  (setq word (substring word (1+ (match-end 1)))))
 	(cond ((string-match "^<<.+>>$" word)
 	       (setq key (vconcat (if (eq (key-binding [?\M-x])
@@ -665,30 +672,31 @@ If START or END is negative, it counts from the end."
 	      (t
 	       (let ((orig-word word) (prefix 0) (bits 0))
 		 (while (string-match "^[ACHMsS]-." word)
-		   (incf bits (cdr (assq (aref word 0)
+		   (cl-incf bits (cdr (assq (aref word 0)
 					 '((?A . ?\A-\^@) (?C . ?\C-\^@)
 					   (?H . ?\H-\^@) (?M . ?\M-\^@)
 					   (?s . ?\s-\^@) (?S . ?\S-\^@)))))
-		   (incf prefix 2)
-		   (callf substring word 2))
+		   (cl-incf prefix 2)
+		   (cl-callf substring word 2))
 		 (when (string-match "^\\^.$" word)
-		   (incf bits ?\C-\^@)
-		   (incf prefix)
-		   (callf substring word 1))
+		   (cl-incf bits ?\C-\^@)
+		   (cl-incf prefix)
+		   (cl-callf substring word 1))
 		 (let ((found (assoc word '(("NUL" . "\0") ("RET" . "\r")
 					    ("LFD" . "\n") ("TAB" . "\t")
 					    ("ESC" . "\e") ("SPC" . " ")
 					    ("DEL" . "\177")))))
 		   (when found (setq word (cdr found))))
 		 (when (string-match "^\\\\[0-7]+$" word)
-		   (loop for ch across word
-			 for n = 0 then (+ (* n 8) ch -48)
-			 finally do (setq word (vector n))))
+		   (cl-loop for ch across word
+                            for n = 0 then (+ (* n 8) ch -48)
+                            finally do (setq word (vector n))))
 		 (cond ((= bits 0)
 			(setq key word))
 		       ((and (= bits ?\M-\^@) (stringp word)
 			     (string-match "^-?[0-9]+$" word))
-			(setq key (loop for x across word collect (+ x bits))))
+			(setq key (cl-loop for x across word
+                                           collect (+ x bits))))
 		       ((/= (length word) 1)
 			(error "%s must prefix a single character, not %s"
 			       (substring orig-word 0 prefix) word))
@@ -702,21 +710,21 @@ If START or END is negative, it counts from the end."
 		       (t
 			(setq key (list (+ bits (aref word 0)))))))))
 	(when key
-	  (loop repeat times do (callf vconcat res key)))))
+	  (cl-loop repeat times do (cl-callf vconcat res key)))))
     (when (and (>= (length res) 4)
 	       (eq (aref res 0) ?\C-x)
 	       (eq (aref res 1) ?\()
 	       (eq (aref res (- (length res) 2)) ?\C-x)
 	       (eq (aref res (- (length res) 1)) ?\)))
-      (setq res (edmacro-subseq res 2 -2)))
+      (setq res (cl-subseq res 2 -2)))
     (if (and (not need-vector)
-	     (loop for ch across res
-		   always (and (char-valid-p ch)
-			       (let ((ch2 (logand ch (lognot ?\M-\^@))))
-				 (and (>= ch2 0) (<= ch2 127))))))
-	(concat (loop for ch across res
-		      collect (if (= (logand ch ?\M-\^@) 0)
-				  ch (+ ch 128))))
+	     (cl-loop for ch across res
+                      always (and (characterp ch)
+                                  (let ((ch2 (logand ch (lognot ?\M-\^@))))
+                                    (and (>= ch2 0) (<= ch2 127))))))
+	(concat (cl-loop for ch across res
+                         collect (if (= (logand ch ?\M-\^@) 0)
+                                     ch (+ ch 128))))
       res)))
 
 (provide 'edmacro)
