@@ -1,7 +1,7 @@
-;;; chart.el --- Draw charts (bar charts, etc)
+;;; chart.el --- Draw charts (bar charts, etc)  -*- lexical-binding: t -*-
 
-;; Copyright (C) 1996, 1998, 1999, 2001, 2004, 2005, 2007, 2008, 2009, 2010, 2011, 2012
-;;   Free Software Foundation, Inc.
+;; Copyright (C) 1996, 1998-1999, 2001, 2004-2005, 2007-2015 Free
+;; Software Foundation, Inc.
 
 ;; Author: Eric M. Ludlam  <zappo@gnu.org>
 ;; Version: 0.2
@@ -62,21 +62,13 @@
 (require 'eieio)
 
 ;;; Code:
-(defvar chart-map nil "Keymap used in chart mode.")
-(if chart-map
-    ()
-  (setq chart-map (make-sparse-keymap))
-  )
+(define-obsolete-variable-alias 'chart-map 'chart-mode-map "24.1")
+(defvar chart-mode-map (make-sparse-keymap) "Keymap used in chart mode.")
 
 (defvar chart-local-object nil
   "Local variable containing the locally displayed chart object.")
 (make-variable-buffer-local 'chart-local-object)
 
-(defvar chart-face-list nil
-  "Faces used to colorize charts.
-List is limited currently, which is ok since you really can't display
-too much in text characters anyways.")
-
 (defvar chart-face-color-list '("red" "green" "blue"
 				"cyan" "yellow" "purple")
   "Colors to use when generating `chart-face-list'.
@@ -90,45 +82,46 @@ Colors will be the background color.")
 Useful if new Emacs is used on B&W display.")
 
 (defcustom chart-face-use-pixmaps nil
-  "*Non-nil to use fancy pixmaps in the background of chart face colors."
+  "Non-nil to use fancy pixmaps in the background of chart face colors."
   :group 'eieio
   :type 'boolean)
 
-(if (and (if (fboundp 'display-color-p)
-	     (display-color-p)
-	   window-system)
-	 (not chart-face-list))
-    (let ((cl chart-face-color-list)
-	  (pl chart-face-pixmap-list)
-	  nf)
-      (while cl
-	(setq nf (make-face (intern (concat "chart-" (car cl) "-" (car pl)))))
-	(if (condition-case nil
-		(> (x-display-color-cells) 4)
-	      (error t))
-	    (set-face-background nf (car cl))
-	  (set-face-background nf "white"))
-	(set-face-foreground nf "black")
-	(if (and chart-face-use-pixmaps
-		 pl
-		 (fboundp 'set-face-background-pixmap))
-	    (condition-case nil
-		(set-face-background-pixmap nf (car pl))
-	      (error (message "Cannot set background pixmap %s" (car pl)))))
-	(setq chart-face-list (cons nf chart-face-list))
-	(setq cl (cdr cl)
-	      pl (cdr pl)))))
+(declare-function x-display-color-cells "xfns.c" (&optional terminal))
 
-(defun chart-mode ()
+(defvar chart-face-list
+  (if (display-color-p)
+      (let ((cl chart-face-color-list)
+            (pl chart-face-pixmap-list)
+            (faces ())
+            nf)
+        (while cl
+          (setq nf (make-face
+                    (intern (concat "chart-" (car cl) "-" (car pl)))))
+          (set-face-background nf (if (condition-case nil
+                                          (> (x-display-color-cells) 4)
+                                        (error t))
+                                      (car cl)
+                                    "white"))
+          (set-face-foreground nf "black")
+          (if (and chart-face-use-pixmaps
+                   pl
+                   (fboundp 'set-face-background-pixmap))
+              (condition-case nil
+                  (set-face-background-pixmap nf (car pl))
+                (error (message "Cannot set background pixmap %s" (car pl)))))
+          (push nf faces)
+          (setq cl (cdr cl)
+                pl (cdr pl)))
+        faces))
+  "Faces used to colorize charts.
+List is limited currently, which is ok since you really can't display
+too much in text characters anyways.")
+
+(define-derived-mode chart-mode fundamental-mode "CHART"
   "Define a mode in Emacs for displaying a chart."
-  (kill-all-local-variables)
-  (use-local-map chart-map)
-  (setq major-mode 'chart-mode
-	mode-name "CHART")
   (buffer-disable-undo)
   (set (make-local-variable 'font-lock-global-modes) nil)
-  (font-lock-mode -1)
-  (run-hooks 'chart-mode-hook)
+  (font-lock-mode -1)                   ;Isn't it off already?  --Stef
   )
 
 (defun chart-new-buffer (obj)
@@ -163,7 +156,7 @@ Returns the newly created buffer."
    )
   "Superclass for all charts to be displayed in an Emacs buffer.")
 
-(defmethod initialize-instance :AFTER ((obj chart) &rest fields)
+(defmethod initialize-instance :AFTER ((obj chart) &rest _fields)
   "Initialize the chart OBJ being created with FIELDS.
 Make sure the width/height is correct."
   (oset obj x-width (- (window-width) 10))
@@ -176,7 +169,7 @@ Make sure the width/height is correct."
 	      :initform t)
    (name-face :initarg :name-face
 	      :initform 'bold)
-   (labels-face :initarg :lables-face
+   (labels-face :initarg :labels-face
 		:initform 'italic)
    (chart :initarg :chart
 	  :initform nil)
@@ -283,7 +276,7 @@ START and END represent the boundary."
 		       (float (- (cdr range) (car range)))))))))
   )
 
-(defmethod chart-axis-draw ((a chart-axis-range) &optional dir margin zone start end)
+(defmethod chart-axis-draw ((a chart-axis-range) &optional dir margin zone _start _end)
   "Draw axis information based upon a range to be spread along the edge.
 A is the chart to draw.  DIR is the direction.
 MARGIN, ZONE, START, and END specify restrictions in chart space."
@@ -336,7 +329,7 @@ Automatically compensates for direction."
 	  (+ m -1 (round (* lpn (+ 1.0 (float n))))))
     ))
 
-(defmethod chart-axis-draw ((a chart-axis-names) &optional dir margin zone start end)
+(defmethod chart-axis-draw ((a chart-axis-names) &optional dir margin zone _start _end)
   "Draw axis information based upon A range to be spread along the edge.
 Optional argument DIR is the direction of the chart.
 Optional arguments MARGIN, ZONE, START and END specify boundaries of the drawing."
@@ -477,7 +470,7 @@ See `chart-sort-matchlist' for more details."
 	  (progn
 	    (chart-sort-matchlist s2 s1 pred)
 	    (setq s (oref s2 data)))
-	(error "Sorting of chart %s not supported" (object-name c))))
+	(error "Sorting of chart %s not supported" (eieio-object-name c))))
     (if (eq (oref c direction) 'horizontal)
 	(oset (oref c y-axis) items s)
       (oset (oref c x-axis) items s)
@@ -529,9 +522,9 @@ cons cells of the form (NAME . NUM).  See `sort' for more details."
 (defun chart-zap-chars (n)
   "Zap up to N chars without deleting EOLs."
   (if (not (eobp))
-      (if (< n (- (save-excursion (end-of-line) (point)) (point)))
+      (if (< n (- (point-at-eol) (point)))
 	  (delete-char n)
-	(delete-region (point) (save-excursion (end-of-line) (point))))))
+	(delete-region (point) (point-at-eol)))))
 
 (defun chart-display-label (label dir zone start end &optional face)
   "Display LABEL in direction DIR in column/row ZONE between START and END.
@@ -641,12 +634,12 @@ SORT-PRED if desired."
 	    (setq extlst (cons s extlst)
 		  cntlst (cons 1 cntlst)))))
       (setq flst (cdr flst)))
-    ;; Lets create the chart!
+    ;; Let's create the chart!
     (chart-bar-quickie 'vertical "Files Extension Distribution"
 		       extlst "File Extensions"
 		       cntlst "# of occurrences"
 		       10
-		       '(lambda (a b) (> (cdr a) (cdr b))))
+		       (lambda (a b) (> (cdr a) (cdr b))))
     ))
 
 (defun chart-space-usage (d)
@@ -676,34 +669,20 @@ SORT-PRED if desired."
 		       nmlst "File Name"
 		       cntlst "File Size"
 		       10
-		       '(lambda (a b) (> (cdr a) (cdr b))))
+		       (lambda (a b) (> (cdr a) (cdr b))))
     ))
 
 (defun chart-emacs-storage ()
   "Chart the current storage requirements of Emacs."
   (interactive)
-  (let* ((data (garbage-collect))
-	 (names '("strings/2" "vectors"
-		  "conses" "free cons"
-		  "syms" "free syms"
-		  "markers" "free mark"
-		  ;; "floats" "free flt"
-		  ))
-	 (nums (list (/ (nth 3 data) 2)
-		     (nth 4 data)
-		     (car (car data))	; conses
-		     (cdr (car data))
-		     (car (nth 1 data)) ; syms
-		     (cdr (nth 1 data))
-		     (car (nth 2 data))	; markers
-		     (cdr (nth 2 data))
-		     ;(car (nth 5 data)) ; floats are Emacs only
-		     ;(cdr (nth 5 data))
-		     )))
-    ;; Lets create the chart!
+  (let* ((data (garbage-collect)))
+    ;; Let's create the chart!
     (chart-bar-quickie 'vertical "Emacs Runtime Storage Usage"
-		       names "Storage Items"
-		       nums "Objects")))
+		       (mapcar (lambda (x) (symbol-name (car x))) data)
+                       "Storage Items"
+		       (mapcar (lambda (x) (* (nth 1 x) (nth 2 x)))
+                               data)
+                       "Bytes")))
 
 (defun chart-emacs-lists ()
   "Chart out the size of various important lists."
@@ -717,7 +696,7 @@ SORT-PRED if desired."
     (if (fboundp 'x-display-list)
 	(setq names (append names '("x-displays"))
 	      nums (append nums (list (length (x-display-list))))))
-    ;; Lets create the chart!
+    ;; Let's create the chart!
     (chart-bar-quickie 'vertical "Emacs List Size Chart"
 		       names "Various Lists"
 		       nums "Objects")))
@@ -744,11 +723,10 @@ SORT-PRED if desired."
 		       nmlst "User Names"
 		       cntlst "# of occurrences"
 		       10
-		       '(lambda (a b) (> (cdr a) (cdr b))))
+		       (lambda (a b) (> (cdr a) (cdr b))))
     ))
 
 
 (provide 'chart)
 
-;; arch-tag: 43847e44-5b45-465e-adc9-e505490a6b59
 ;;; chart.el ends here
