@@ -1,6 +1,6 @@
 ;;; find-cmd.el --- Build a valid find(1) command with sexps
 
-;; Copyright (C) 2008, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
+;; Copyright (C) 2008-2015 Free Software Foundation, Inc.
 
 ;; Author: Philip Jackson <phil@shellarchive.co.uk>
 ;; Version: 0.6
@@ -63,6 +63,7 @@
     (cnewer     . (1))
     (ctime      . (1))
     (empty      . (0))
+    (executable . (0))
     (false      . (0))
     (fstype     . (1))
     (gid        . (1))
@@ -70,37 +71,43 @@
     (ilname     . (1))
     (iname      . (1))
     (inum       . (1))
-    (iwholename . (1))
+    (ipath      . (1))
     (iregex     . (1))
+    (iwholename . (1))
     (links      . (1))
     (lname      . (1))
     (mmin       . (1))
     (mtime      . (1))
     (name       . (1))
     (newer      . (1))
-    (nouser     . (0))
     (nogroup    . (0))
+    (nouser     . (0))
     (path       . (1))
     (perm       . (0))
+    (readable   . (0))
     (regex      . (1))
-    (wholename  . (1))
+    (samefile   . (1))
     (size       . (1))
     (true       . (0))
     (type       . (1))
     (uid        . (1))
     (used       . (1))
     (user       . (1))
+    (wholename  . (1))
+    (writable   . (0))
     (xtype      . (nil))
 
     ;; normal options (always true)
+    (daystart              . (0))
     (depth                 . (0))
     (maxdepth              . (1))
     (mindepth              . (1))
     (mount                 . (0))
     (noleaf                . (0))
-    (xdev                  . (0))
     (ignore_readdir_race   . (0))
     (noignore_readdir_race . (0))
+    (regextype             . (1))
+    (xdev                  . (0))
 
     ;; actions
     (delete  . (0))
@@ -240,5 +247,4 @@ them into valid switches.  The result is -and(ed) together."
 
 (provide 'find-cmd)
 
-;; arch-tag: 9687fd9e-4e90-4022-864a-f904526e2046
 ;;; find-cmd.el ends here
