@@ -1,9 +1,9 @@
 ;;; help-macro.el --- makes command line help such as help-for-help
 
-;; Copyright (C) 1993-1994, 2001-2012 Free Software Foundation, Inc.
+;; Copyright (C) 1993-1994, 2001-2015 Free Software Foundation, Inc.
 
 ;; Author: Lynn Slater <lrs@indetech.com>
-;; Maintainer: FSF
+;; Maintainer: emacs-devel@gnu.org
 ;; Created: Mon Oct  1 11:42:39 1990
 ;; Adapted-By: ESR
 ;; Package: emacs
@@ -69,6 +69,10 @@
 
 (require 'backquote)
 
+;; This needs to be autoloaded because it is used in the
+;; make-help-screen macro.  Using (bound-and-true-p three-step-help)
+;; is not an acceptable alternative, because nothing loads help-macro
+;; in a normal session, so any user customization would never be applied.
 ;;;###autoload
 (defcustom three-step-help nil
   "Non-nil means give more info about Help command in three steps.
@@ -130,11 +134,11 @@ and then returns."
 		   (when (or (eq char ??) (eq char help-char)
 			     (memq char help-event-list))
 		     (setq config (current-window-configuration))
-		     (switch-to-buffer-other-window "*Help*")
+		     (pop-to-buffer " *Metahelp*" nil t)
 		     (and (fboundp 'make-frame)
-			  (not (eq (window-frame (selected-window))
+			  (not (eq (window-frame)
 				   prev-frame))
-			  (setq new-frame (window-frame (selected-window))
+			  (setq new-frame (window-frame)
 				config nil))
 		     (setq buffer-read-only nil)
 		     (let ((inhibit-read-only t))
