@@ -1,16 +1,16 @@
-;;; cpp.el --- Highlight or hide text according to cpp conditionals.
+;;; cpp.el --- highlight or hide text according to cpp conditionals
 
-;; Copyright (C) 1994, 1995 Free Software Foundation
+;; Copyright (C) 1994-1995, 2001-2015 Free Software Foundation, Inc.
 
 ;; Author: Per Abrahamsen <abraham@dina.kvl.dk>
 ;; Keywords: c, faces, tools
 
 ;; This file is part of GNU Emacs.
 
-;; GNU Emacs is free software; you can redistribute it and/or modify
+;; GNU Emacs is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; the Free Software Foundation, either version 3 of the License, or
+;; (at your option) any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,9 +18,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-;; Boston, MA 02111-1307, USA.
+;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -32,7 +30,7 @@
 ;;; Todo:
 
 ;; Should parse "#if" and "#elif" expressions and merge the faces
-;; somehow. 
+;; somehow.
 
 ;; Somehow it is sometimes possible to make changes near a read only
 ;; area which you can't undo.  Their are other strange effects in that
@@ -49,39 +47,45 @@
 ;;; Customization:
 (defgroup cpp nil
   "Highlight or hide text according to cpp conditionals."
-  :group 'C
+  :link '(custom-group-link :tag "Font Lock Faces group" font-lock-faces)
+  :group 'c
   :prefix "cpp-")
 
 (defcustom cpp-config-file (convert-standard-filename ".cpp.el")
-  "*File name to save cpp configuration."
+  "File name to save cpp configuration."
   :type 'file
   :group 'cpp)
 
+(define-widget 'cpp-face 'lazy
+  "Either a face or the special symbol 'invisible'."
+  :type '(choice (const invisible) (face)))
+
 (defcustom cpp-known-face 'invisible
-  "*Face used for known cpp symbols."
-  :type 'face
+  "Face used for known cpp symbols."
+  :type 'cpp-face
   :group 'cpp)
 
 (defcustom cpp-unknown-face 'highlight
-  "*Face used for unknown cpp symbols."
-  :type 'face
+  "Face used for unknown cpp symbols."
+  :type 'cpp-face
   :group 'cpp)
 
-(defcustom cpp-face-type 'light 
-  "*Indicate what background face type you prefer.
+(defcustom cpp-face-type 'light
+  "Indicate what background face type you prefer.
 Can be either light or dark for color screens, mono for monochrome
-screens, and none if you don't use a window system."
+screens, and none if you don't use a window system and don't have
+a color-capable display."
   :options '(light dark mono nil)
   :type 'symbol
   :group 'cpp)
 
 (defcustom cpp-known-writable t
-  "*Non-nil means you are allowed to modify the known conditionals."
+  "Non-nil means you are allowed to modify the known conditionals."
   :type 'boolean
   :group 'cpp)
 
 (defcustom cpp-unknown-writable t
-  "*Non-nil means you are allowed to modify the unknown conditionals."
+  "Non-nil means you are allowed to modify the unknown conditionals."
   :type 'boolean
   :group 'cpp)
 
@@ -91,11 +95,13 @@ Each entry is a list with the following elements:
 0. The name of the macro (a string).
 1. Face used for text that is `ifdef' the macro.
 2. Face used for text that is `ifndef' the macro.
-3. `t', `nil', or `both' depending on what text may be edited."
-  :type '(repeat (list string face face
-		       (choice (const t)
-			       (const nil)
-			       (const both))))
+3. t, nil, or `both' depending on what text may be edited."
+  :type '(repeat (list (string :tag "Macro")
+		       (cpp-face :tag "True")
+		       (cpp-face :tag "False")
+		       (choice (const :tag "True branch writable" t)
+			       (const :tag "False branch writable" nil)
+			       (const :tag "Both branches writable" both))))
   :group 'cpp)
 
 (defvar cpp-overlay-list nil)
@@ -130,20 +136,25 @@ Each entry is a list with the following elements:
     ("true" . t)
     ("both" . both)))
 
+;; FIXME Gets clobbered by cpp-choose-face, so why is even it a defcustom?
 (defcustom cpp-face-default-list nil
   "Alist of faces you can choose from for cpp conditionals.
 Each element has the form (STRING . FACE), where STRING
 serves as a name (for `cpp-highlight-buffer' only)
 and FACE is either a face (a symbol)
 or a cons cell (background-color . COLOR)."
-  :type '(repeat (cons string (choice face (cons (const background-color) string))))
+  :type '(alist :key-type (string :tag "Name")
+		:value-type (choice face
+				    (const invisible)
+				    (cons (const background-color)
+					  (string :tag "Color"))))
   :group 'cpp)
 
 (defcustom cpp-face-light-name-list
   '("light gray" "light blue" "light cyan" "light yellow" "light pink"
     "pale green" "beige" "orange" "magenta" "violet" "medium purple"
     "turquoise")
-  "Background colours useful with dark foreground colors."
+  "Background colors useful with dark foreground colors."
   :type '(repeat string)
   :group 'cpp)
 
@@ -151,18 +162,20 @@ or a cons cell (background-color . COLOR)."
   '("dim gray" "blue" "cyan" "yellow" "red"
     "dark green" "brown" "dark orange" "dark khaki" "dark violet" "purple"
     "dark turquoise")
-  "Background colours useful with light foreground colors."
+  "Background colors useful with light foreground colors."
   :type '(repeat string)
   :group 'cpp)
 
 (defcustom cpp-face-light-list nil
   "Alist of names and faces to be used for light backgrounds."
-  :type '(repeat (cons string face))
+  :type '(repeat (cons string (choice face
+				      (cons (const background-color) string))))
   :group 'cpp)
 
 (defcustom cpp-face-dark-list nil
   "Alist of names and faces to be used for dark backgrounds."
-  :type '(repeat (cons string face))
+  :type '(repeat (cons string (choice face
+				      (cons (const background-color) string))))
   :group 'cpp)
 
 (defcustom cpp-face-mono-list
@@ -178,7 +191,7 @@ or a cons cell (background-color . COLOR)."
    '(("default" . default)
      ("invisible" . invisible))
    "Alist of names and faces available even if you don't use a window system."
-  :type '(repeat (cons string face))
+  :type '(repeat (cons string cpp-face))
   :group 'cpp)
 
 (defvar cpp-face-all-list
@@ -300,7 +313,6 @@ A prefix arg suppresses display of that buffer."
   ;; Pop top of cpp-state-stack and create overlay.
   (let ((entry (assoc (nth 1 (car cpp-state-stack)) cpp-edit-list))
 	(branch (nth 0 (car cpp-state-stack)))
-	(begin (nth 2 (car cpp-state-stack)))
 	(end (nth 3 (car cpp-state-stack))))
     (setq cpp-state-stack (cdr cpp-state-stack))
     (if entry
@@ -365,7 +377,7 @@ A prefix arg suppresses display of that buffer."
     (cond ((eq cpp-unknown-face 'invisible)
 	   (cpp-make-overlay-hidden overlay))
 	  ((eq cpp-unknown-face 'default))
-	  (t 
+	  (t
 	   (overlay-put overlay 'face cpp-unknown-face)))
     (if cpp-unknown-writable
 	()
@@ -376,9 +388,6 @@ A prefix arg suppresses display of that buffer."
 (defun cpp-make-overlay-hidden (overlay)
   ;; Make overlay hidden and intangible.
   (overlay-put overlay 'invisible 'cpp)
-  (overlay-put overlay 'intangible t)
-  ;; Unfortunately `intangible' is not implemented for overlays yet,
-  ;; so we make is read-only instead.
   (overlay-put overlay 'modification-hooks '(cpp-signal-read-only))
   (overlay-put overlay 'insert-in-front-hooks '(cpp-signal-read-only)))
 
@@ -393,7 +402,7 @@ A prefix arg suppresses display of that buffer."
   (overlay-put overlay 'insert-in-front-hooks '(cpp-grow-overlay))
   (overlay-put overlay 'insert-behind-hooks '(cpp-grow-overlay)))
 
-(defun cpp-signal-read-only (overlay after start end &optional len)
+(defun cpp-signal-read-only (overlay after start end &optional _len)
   ;; Only allow deleting the whole overlay.
   ;; Trying to change a read-only overlay.
   (if (and (not after)
@@ -401,7 +410,7 @@ A prefix arg suppresses display of that buffer."
 	       (> (overlay-end overlay) end)))
       (error "This text is read only")))
 
-(defun cpp-grow-overlay (overlay after start end &optional len)
+(defun cpp-grow-overlay (overlay after start end &optional _len)
   ;; Make OVERLAY grow to contain range START to END.
   (if after
       (move-overlay overlay
@@ -410,63 +419,60 @@ A prefix arg suppresses display of that buffer."
 
 ;;; Edit Buffer:
 
-(defvar cpp-edit-map nil)
-;; Keymap for `cpp-edit-mode'.
+(defvar cpp-edit-mode-map
+  (let ((map (make-keymap)))
+    (suppress-keymap map)
+    (define-key map [ down-mouse-2 ] 'cpp-push-button)
+    (define-key map [ mouse-2 ] 'ignore)
+    (define-key map " " 'scroll-up-command)
+    (define-key map [?\S-\ ] 'scroll-down-command)
+    (define-key map "\C-?" 'scroll-down-command)
+    (define-key map [ delete ] 'scroll-down)
+    (define-key map "\C-c\C-c" 'cpp-edit-apply)
+    (define-key map "a" 'cpp-edit-apply)
+    (define-key map "A" 'cpp-edit-apply)
+    (define-key map "r" 'cpp-edit-reset)
+    (define-key map "R" 'cpp-edit-reset)
+    (define-key map "s" 'cpp-edit-save)
+    (define-key map "S" 'cpp-edit-save)
+    (define-key map "l" 'cpp-edit-load)
+    (define-key map "L" 'cpp-edit-load)
+    (define-key map "h" 'cpp-edit-home)
+    (define-key map "H" 'cpp-edit-home)
+    (define-key map "b" 'cpp-edit-background)
+    (define-key map "B" 'cpp-edit-background)
+    (define-key map "k" 'cpp-edit-known)
+    (define-key map "K" 'cpp-edit-known)
+    (define-key map "u" 'cpp-edit-unknown)
+    (define-key map "u" 'cpp-edit-unknown)
+    (define-key map "t" 'cpp-edit-true)
+    (define-key map "T" 'cpp-edit-true)
+    (define-key map "f" 'cpp-edit-false)
+    (define-key map "F" 'cpp-edit-false)
+    (define-key map "w" 'cpp-edit-write)
+    (define-key map "W" 'cpp-edit-write)
+    (define-key map "X" 'cpp-edit-toggle-known)
+    (define-key map "x" 'cpp-edit-toggle-known)
+    (define-key map "Y" 'cpp-edit-toggle-unknown)
+    (define-key map "y" 'cpp-edit-toggle-unknown)
+    (define-key map "q" 'bury-buffer)
+    (define-key map "Q" 'bury-buffer)
+    map)
+  "Keymap for `cpp-edit-mode'.")
+
 
-(if cpp-edit-map
-    ()
-  (setq cpp-edit-map (make-keymap))
-  (suppress-keymap cpp-edit-map)
-  (define-key cpp-edit-map [ down-mouse-2 ] 'cpp-push-button)
-  (define-key cpp-edit-map [ mouse-2 ] 'ignore)
-  (define-key cpp-edit-map " " 'scroll-up)
-  (define-key cpp-edit-map "\C-?" 'scroll-down)
-  (define-key cpp-edit-map [ delete ] 'scroll-down)
-  (define-key cpp-edit-map "\C-c\C-c" 'cpp-edit-apply)
-  (define-key cpp-edit-map "a" 'cpp-edit-apply)
-  (define-key cpp-edit-map "A" 'cpp-edit-apply)
-  (define-key cpp-edit-map "r" 'cpp-edit-reset)
-  (define-key cpp-edit-map "R" 'cpp-edit-reset)
-  (define-key cpp-edit-map "s" 'cpp-edit-save)
-  (define-key cpp-edit-map "S" 'cpp-edit-save)
-  (define-key cpp-edit-map "l" 'cpp-edit-load)
-  (define-key cpp-edit-map "L" 'cpp-edit-load)
-  (define-key cpp-edit-map "h" 'cpp-edit-home)
-  (define-key cpp-edit-map "H" 'cpp-edit-home)
-  (define-key cpp-edit-map "b" 'cpp-edit-background)
-  (define-key cpp-edit-map "B" 'cpp-edit-background)
-  (define-key cpp-edit-map "k" 'cpp-edit-known)
-  (define-key cpp-edit-map "K" 'cpp-edit-known)
-  (define-key cpp-edit-map "u" 'cpp-edit-unknown)
-  (define-key cpp-edit-map "u" 'cpp-edit-unknown)
-  (define-key cpp-edit-map "t" 'cpp-edit-true)
-  (define-key cpp-edit-map "T" 'cpp-edit-true)
-  (define-key cpp-edit-map "f" 'cpp-edit-false)
-  (define-key cpp-edit-map "F" 'cpp-edit-false)
-  (define-key cpp-edit-map "w" 'cpp-edit-write)
-  (define-key cpp-edit-map "W" 'cpp-edit-write)
-  (define-key cpp-edit-map "X" 'cpp-edit-toggle-known)
-  (define-key cpp-edit-map "x" 'cpp-edit-toggle-known)
-  (define-key cpp-edit-map "Y" 'cpp-edit-toggle-unknown)
-  (define-key cpp-edit-map "y" 'cpp-edit-toggle-unknown)
-  (define-key cpp-edit-map "q" 'bury-buffer)
-  (define-key cpp-edit-map "Q" 'bury-buffer))
 
 (defvar cpp-edit-symbols nil)
 ;; Symbols defined in the edit buffer.
 (make-variable-buffer-local 'cpp-edit-symbols)
 
-(defun cpp-edit-mode ()
+(define-derived-mode cpp-edit-mode fundamental-mode "CPP Edit"
   "Major mode for editing the criteria for highlighting cpp conditionals.
-Click on objects to change them.  
+Click on objects to change them.
 You can also use the keyboard accelerators indicated like this: [K]ey."
-  (kill-all-local-variables)
   (buffer-disable-undo)
   (auto-save-mode -1)
-  (setq buffer-read-only t)
-  (setq major-mode 'cpp-edit-mode)
-  (setq mode-name "CPP Edit")
-  (use-local-map cpp-edit-map))
+  (setq buffer-read-only t))
 
 (defun cpp-edit-apply ()
   "Apply edited display information to original buffer."
@@ -518,6 +524,8 @@ You can also use the keyboard accelerators indicated like this: [K]ey."
 		     'cpp-edit-toggle-unknown)
     (insert (format "\n\n\n%39s: %14s %14s %7s\n\n" "Expression"
 		    "[T]rue Face" "[F]alse Face" "[W]rite"))
+
+    (setq symbols (reverse symbols))
     (while symbols
       (let*  ((symbol (car symbols))
 	      (entry (assoc symbol cpp-edit-list))
@@ -532,7 +540,7 @@ You can also use the keyboard accelerators indicated like this: [K]ey."
 		 (eq write 'both))
 	    (setq cpp-edit-list (delq entry cpp-edit-list)
 		  entry nil))
-	
+
 	(if (> (length symbol) 39)
 	    (insert (substring symbol 0 39) ": ")
 	  (insert (format "%39s: " symbol)))
@@ -560,15 +568,14 @@ You can also use the keyboard accelerators indicated like this: [K]ey."
 	 (load-file cpp-config-file))
 	((file-readable-p (concat "~/" cpp-config-file))
 	 (load-file cpp-config-file)))
-  (if (eq major-mode 'cpp-edit-mode)
+  (if (derived-mode-p 'cpp-edit-mode)
       (cpp-edit-reset)))
 
 (defun cpp-edit-save ()
   "Save the current cpp configuration in a file."
   (interactive)
   (require 'pp)
-  (save-excursion
-    (set-buffer cpp-edit-buffer)
+  (with-current-buffer cpp-edit-buffer
     (let ((buffer (find-file-noselect cpp-config-file)))
       (set-buffer buffer)
       (erase-buffer)
@@ -613,7 +620,8 @@ You can also use the keyboard accelerators indicated like this: [K]ey."
 
 (defun cpp-edit-toggle-known (arg)
   "Toggle writable status for known conditionals.
-With optional argument ARG, make them writable iff ARG is positive."
+With optional argument ARG, make them writable if ARG is positive,
+otherwise make them unwritable."
   (interactive "@P")
   (if (or (and (null arg) cpp-known-writable)
 	  (<= (prefix-numeric-value arg) 0))
@@ -623,7 +631,8 @@ With optional argument ARG, make them writable iff ARG is positive."
 
 (defun cpp-edit-toggle-unknown (arg)
   "Toggle writable status for unknown conditionals.
-With optional argument ARG, make them writable iff ARG is positive."
+With optional argument ARG, make them writable if ARG is positive,
+otherwise make them unwritable."
   (interactive "@P")
   (if (or (and (null arg) cpp-unknown-writable)
 	  (<= (prefix-numeric-value arg) 0))
@@ -646,7 +655,7 @@ With optional argument ARG, make them writable iff ARG is positive."
   (interactive
    (let ((symbol (cpp-choose-symbol)))
      (list symbol
-	   (cpp-choose-face "False face" 
+	   (cpp-choose-face "False face"
 			    (nth 2 (assoc symbol cpp-edit-list))))))
   (setcar (nthcdr 2 (cpp-edit-list-entry-get-or-create symbol)) face)
   (cpp-edit-reset))
@@ -673,7 +682,7 @@ BRANCH should be either nil (false branch), t (true branch) or 'both."
   ;; Choose a symbol if called from keyboard, otherwise use the one clicked on.
   (if cpp-button-event
       cpp-callback-data
-    (completing-read "Symbol: " (mapcar 'list cpp-edit-symbols) nil t)))
+    (completing-read "Symbol: " cpp-edit-symbols nil t)))
 
 (defun cpp-choose-branch ()
   ;; Choose a branch, either nil, t, or both.
@@ -684,7 +693,7 @@ BRANCH should be either nil (false branch), t (true branch) or 'both."
 		cpp-branch-list))))
 
 (defun cpp-choose-face (prompt default)
-  ;; Choose a face from cpp-face-defalt-list.
+  ;; Choose a face from cpp-face-default-list.
   ;; PROMPT is what to say to the user.
   ;; DEFAULT is the default face.
   (or (if cpp-button-event
@@ -773,6 +782,7 @@ BRANCH should be either nil (false branch), t (true branch) or 'both."
     (add-text-properties from to
 			 (append (list 'face face)
 				 '(mouse-face highlight)
+				 '(help-echo "mouse-2: change/use this item")
 				 (list 'cpp-callback callback)
 				 (if data (list 'cpp-data data))))))
 
@@ -795,7 +805,8 @@ BRANCH should be either nil (false branch), t (true branch) or 'both."
   ;; Create entry for face with background COLOR.
   (cons color (cons 'background-color color)))
 
-(cpp-choose-default-face (if window-system cpp-face-type 'none))
+(cpp-choose-default-face
+ (if (or window-system (display-color-p)) cpp-face-type 'none))
 
 (defun cpp-face-name (face)
   ;; Return the name of FACE from `cpp-face-all-list'.
