@@ -1,6 +1,6 @@
 ;;; xt-mouse.el --- support the mouse when emacs run in an xterm
 
-;; Copyright (C) 1994, 2000-2013 Free Software Foundation, Inc.
+;; Copyright (C) 1994, 2000-2015 Free Software Foundation, Inc.
 
 ;; Author: Per Abrahamsen <abraham@dina.kvl.dk>
 ;; Keywords: mouse, terminals
@@ -63,8 +63,8 @@ http://invisible-island.net/xterm/ctlseqs/ctlseqs.html)."
 
 (defun xterm-mouse-translate-1 (&optional extension)
   (save-excursion
-    (save-window-excursion
-      (deactivate-mark)
+    (save-window-excursion              ;FIXME: Why?
+      (deactivate-mark)                 ;FIXME: Why?
       (let* ((xterm-mouse-last nil)
 	     (down (xterm-mouse-event extension))
 	     (down-command (nth 0 down))
@@ -73,10 +73,10 @@ http://invisible-island.net/xterm/ctlseqs/ctlseqs.html)."
 	     (down-binding (key-binding (if (symbolp down-where)
 					    (vector down-where down-command)
 					  (vector down-command))))
-	     (is-click (string-match "^mouse" (symbol-name (car down)))))
+	     (is-down (string-match "down" (symbol-name (car down)))))
 
 	;; Retrieve the expected preface for the up-event.
-	(unless is-click
+	(when is-down
 	  (unless (cond ((null extension)
 			 (and (eq (read-event) ?\e)
 			      (eq (read-event) ?\[)
@@ -88,14 +88,17 @@ http://invisible-island.net/xterm/ctlseqs/ctlseqs.html)."
 	    (error "Unexpected escape sequence from XTerm")))
 
 	;; Process the up-event.
-	(let* ((click (if is-click down (xterm-mouse-event extension)))
+	(let* ((click (if is-down (xterm-mouse-event extension) down))
 	       (click-data  (nth 1 click))
 	       (click-where (nth 1 click-data)))
-	  (if (memq down-binding '(nil ignore))
-	      (if (and (symbolp click-where)
-		       (consp click-where))
-		  (vector (list click-where click-data) click)
-		(vector click))
+          (cond
+           ((null down) nil)
+           ((memq down-binding '(nil ignore))
+            (if (and (symbolp click-where)
+                     (consp click-where))
+                (vector (list click-where click-data) click)
+              (vector click)))
+           (t
 	    (setq unread-command-events
 		  (append (if (eq down-where click-where)
 			      (list click)
@@ -114,7 +117,7 @@ http://invisible-island.net/xterm/ctlseqs/ctlseqs.html)."
 	    (if (and (symbolp down-where)
 		     (consp down-where))
 		(vector (list down-where down-data) down)
-	      (vector down))))))))
+	      (vector down)))))))))
 
 ;; These two variables have been converted to terminal parameters.
 ;;
@@ -135,20 +138,6 @@ http://invisible-island.net/xterm/ctlseqs/ctlseqs.html)."
 		      (terminal-parameter nil 'xterm-mouse-y))))
   pos)
 
-;; Read XTerm sequences above ASCII 127 (#x7f)
-(defun xterm-mouse-event-read ()
-  ;; We get the characters decoded by the keyboard coding system.  Try
-  ;; to recover the raw character.
-  (let ((c (read-event)))
-    (cond ;; If meta-flag is t we get a meta character
-	  ((>= c ?\M-\^@)
-	   (- c (- ?\M-\^@ 128)))
-	  ;; Reencode the character in the keyboard coding system, if
-	  ;; this is a non-ASCII character.
-	  ((>= c #x80)
-	   (aref (encode-coding-string (string c) (keyboard-coding-system)) 0))
-	  (t c))))
-
 (defun xterm-mouse-truncate-wrap (f)
   "Truncate with wrap-around."
   (condition-case nil
@@ -167,40 +156,40 @@ http://invisible-island.net/xterm/ctlseqs/ctlseqs.html)."
 ;; Normal terminal mouse click reporting: expect three bytes, of the
 ;; form <BUTTON+32> <X+32> <Y+32>.  Return a list (EVENT-TYPE X Y).
 (defun xterm-mouse--read-event-sequence-1000 ()
-  (list (let ((code (- (xterm-mouse-event-read) 32)))
-	  (intern
-	   ;; For buttons > 3, the release-event looks differently
-	   ;; (see xc/programs/xterm/button.c, function EditorButton),
-	   ;; and come in a release-event only, no down-event.
-	   (cond ((>= code 64)
-		  (format "mouse-%d" (- code 60)))
-		 ((memq code '(8 9 10))
-		  (setq xterm-mouse-last code)
-		  (format "M-down-mouse-%d" (- code 7)))
-		 ((= code 11)
-		  (format "M-mouse-%d" (- xterm-mouse-last 7)))
-		 ((= code 3)
-		  ;; For buttons > 5 xterm only reports a
-		  ;; button-release event.  Avoid error by mapping
-		  ;; them all to mouse-1.
-		  (format "mouse-%d" (+ 1 (or xterm-mouse-last 0))))
-		 (t
-		  (setq xterm-mouse-last code)
-		  (format "down-mouse-%d" (+ 1 code))))))
-	;; x and y coordinates
-	(- (xterm-mouse-event-read) 33)
-	(- (xterm-mouse-event-read) 33)))
+  (let* ((code (- (read-event) 32))
+         (type
+          ;; For buttons > 3, the release-event looks differently
+          ;; (see xc/programs/xterm/button.c, function EditorButton),
+          ;; and come in a release-event only, no down-event.
+          (cond ((>= code 64)
+                 (format "mouse-%d" (- code 60)))
+                ((memq code '(8 9 10))
+                 (setq xterm-mouse-last (- code 8))
+                 (format "M-down-mouse-%d" (- code 7)))
+                ((and (= code 11) xterm-mouse-last)
+                 (format "M-mouse-%d" (1+ xterm-mouse-last)))
+                ((and (= code 3) xterm-mouse-last)
+                 ;; For buttons > 5 xterm only reports a button-release event.
+                 ;; Drop them since they're not usable and can be spurious.
+                 (format "mouse-%d" (1+ xterm-mouse-last)))
+                ((memq code '(0 1 2))
+                 (setq xterm-mouse-last code)
+                 (format "down-mouse-%d" (+ 1 code)))))
+         (x (- (read-event) 33))
+         (y (- (read-event) 33)))
+    (and type (wholenump x) (wholenump y)
+         (list (intern type) x y))))
 
 ;; XTerm's 1006-mode terminal mouse click reporting has the form
 ;; <BUTTON> ; <X> ; <Y> <M or m>, where the button and ordinates are
 ;; in encoded (decimal) form.  Return a list (EVENT-TYPE X Y).
 (defun xterm-mouse--read-event-sequence-1006 ()
   (let (button-bytes x-bytes y-bytes c)
-    (while (not (eq (setq c (xterm-mouse-event-read)) ?\;))
+    (while (not (eq (setq c (read-event)) ?\;))
       (push c button-bytes))
-    (while (not (eq (setq c (xterm-mouse-event-read)) ?\;))
+    (while (not (eq (setq c (read-event)) ?\;))
       (push c x-bytes))
-    (while (not (memq (setq c (xterm-mouse-event-read)) '(?m ?M)))
+    (while (not (memq (setq c (read-event)) '(?m ?M)))
       (push c y-bytes))
     (list (let* ((code (string-to-number
 			(apply 'string (nreverse button-bytes))))
@@ -236,32 +225,33 @@ which is the \"1006\" extension implemented in Xterm >= 277."
 		      ((eq extension 1006)
 		       (xterm-mouse--read-event-sequence-1006))
 		      (t
-		       (error "Unsupported XTerm mouse protocol"))))
-	 (type (nth 0 click))
-	 (x    (nth 1 click))
-	 (y    (nth 2 click))
-	 ;; Emulate timestamp information.  This is accurate enough
-	 ;; for default value of mouse-1-click-follows-link (450msec).
-	 (timestamp (xterm-mouse-truncate-wrap
-                     (* 1000
-                        (- (float-time)
-                           (or xt-mouse-epoch
-                               (setq xt-mouse-epoch (float-time)))))))
-	 (w (window-at x y))
-         (ltrb (window-edges w))
-         (left (nth 0 ltrb))
-         (top (nth 1 ltrb)))
-    (set-terminal-parameter nil 'xterm-mouse-x x)
-    (set-terminal-parameter nil 'xterm-mouse-y y)
-    (setq
-     last-input-event
-     (list type
-	   (let ((event (if w
-			    (posn-at-x-y (- x left) (- y top) w t)
-			  (append (list nil 'menu-bar)
-				  (nthcdr 2 (posn-at-x-y x y))))))
-	     (setcar (nthcdr 3 event) timestamp)
-	     event)))))
+		       (error "Unsupported XTerm mouse protocol")))))
+    (when click
+      (let* ((type (nth 0 click))
+             (x    (nth 1 click))
+             (y    (nth 2 click))
+             ;; Emulate timestamp information.  This is accurate enough
+             ;; for default value of mouse-1-click-follows-link (450msec).
+             (timestamp (xterm-mouse-truncate-wrap
+                         (* 1000
+                            (- (float-time)
+                               (or xt-mouse-epoch
+                                   (setq xt-mouse-epoch (float-time)))))))
+             (w (window-at x y))
+             (ltrb (window-edges w))
+             (left (nth 0 ltrb))
+             (top (nth 1 ltrb)))
+        (set-terminal-parameter nil 'xterm-mouse-x x)
+        (set-terminal-parameter nil 'xterm-mouse-y y)
+        (setq
+         last-input-event
+         (list type
+               (let ((event (if w
+                                (posn-at-x-y (- x left) (- y top) w t)
+                              (append (list nil 'menu-bar)
+                                      (nthcdr 2 (posn-at-x-y x y))))))
+                 (setcar (nthcdr 3 event) timestamp)
+                 event)))))))
 
 ;;;###autoload
 (define-minor-mode xterm-mouse-mode
