@@ -15,9 +15,10 @@
 
 #include "XMenuInt.h"
 
-XMenuDestroy(display, menu)
-    Display *display;
-    register XMenu *menu;	/* Menu object to destroy. */
+void
+XMenuDestroy(Display *display, register XMenu *menu)
+
+                         	/* Menu object to destroy. */
 {
     register XMPane *p_ptr;	/* Pointer to the current pane. */
     register XMPane *p_next;	/* Pointer to the next pane. */
@@ -114,6 +115,3 @@ XMenuDestroy(display, menu)
      */
     free(menu);
 }
-
-/* arch-tag: 44c9589f-5893-46fc-bc23-1b03a7f9c015
-   (do not change this comment) */
