@@ -1,5 +1,5 @@
 /* Cursor motion subroutines for GNU Emacs.
-   Copyright (C) 1985, 1995, 2001-2012  Free Software Foundation, Inc.
+   Copyright (C) 1985, 1995, 2001-2015 Free Software Foundation, Inc.
     based primarily on public domain code written by Chris Torek
 
 This file is part of GNU Emacs.
@@ -20,7 +20,6 @@ along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include <config.h>
 #include <stdio.h>
-#include <setjmp.h>
 
 #include "lisp.h"
 #include "frame.h"
@@ -29,8 +28,7 @@ along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
 #include "termchar.h"
 #include "tparam.h"
 
-#define	BIG	9999		/* 9999 good on VAXen.  For 16 bit machines
-				   use about 2000.... */
+#define	BIG	9999		/* Good on 32-bit hosts.  */
 
 int cost;		/* sums up costs */
 
@@ -119,7 +117,7 @@ cmcheckmagic (struct tty_display_info *tty)
   if (curX (tty) == FrameCols (tty))
     {
       if (!MagicWrap (tty) || curY (tty) >= FrameRows (tty) - 1)
-	abort ();
+	emacs_abort ();
       if (tty->termscript)
 	putc ('\r', tty->termscript);
       putc ('\r', tty->output);
@@ -216,7 +214,7 @@ calccost (struct tty_display_info *tty,
     if (doit)
       do
           emacs_tputs (tty, p, 1, cmputc);
-      while (0 < --deltay);
+      while (--deltay > 0);
 x:
     if ((deltax = dstx - srcx) == 0)
 	goto done;
@@ -299,7 +297,7 @@ fail:
     if (doit)
       do
           emacs_tputs (tty, p, 1, cmputc);
-      while (0 < --deltax);
+      while (--deltax > 0);
 done:
     return totalcost;
 }
