@@ -1,5 +1,5 @@
 /* GnuTLS glue for GNU Emacs.
-   Copyright (C) 2010-2013 Free Software Foundation, Inc.
+   Copyright (C) 2010-2015 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -42,33 +42,48 @@ typedef enum
   GNUTLS_STAGE_TRANSPORT_POINTERS_SET,
   GNUTLS_STAGE_HANDSHAKE_TRIED,
 
-  GNUTLS_STAGE_READY,
+  GNUTLS_STAGE_READY
 } gnutls_initstage_t;
 
-#define GNUTLS_EMACS_ERROR_NOT_LOADED GNUTLS_E_APPLICATION_ERROR_MIN + 1
+#define GNUTLS_EMACS_ERROR_NOT_LOADED (GNUTLS_E_APPLICATION_ERROR_MIN + 1)
 #define GNUTLS_EMACS_ERROR_INVALID_TYPE GNUTLS_E_APPLICATION_ERROR_MIN
 
 #define GNUTLS_INITSTAGE(proc) (XPROCESS (proc)->gnutls_initstage)
 
-#define GNUTLS_PROCESS_USABLE(proc) (GNUTLS_INITSTAGE(proc) >= GNUTLS_STAGE_READY)
+#define GNUTLS_PROCESS_USABLE(proc) \
+  (GNUTLS_INITSTAGE (proc) >= GNUTLS_STAGE_READY)
 
-#define GNUTLS_LOG(level, max, string) do { if (level <= max) { gnutls_log_function (level, "(Emacs) " string); } } while (0)
+#define GNUTLS_LOG(level, max, string)				\
+  do {								\
+    if ((level) <= (max))					\
+      gnutls_log_function (level, "(Emacs) " string);		\
+  } while (false)
 
-#define GNUTLS_LOG2(level, max, string, extra) do { if (level <= max) { gnutls_log_function2 (level, "(Emacs) " string, extra); } } while (0)
+#define GNUTLS_LOG2(level, max, string, extra)			\
+  do {								\
+    if ((level) <= (max))					\
+      gnutls_log_function2 (level, "(Emacs) " string, extra);	\
+  } while (false)
 
-#define GNUTLS_LOG2i(level, max, string, extra) do { if (level <= max) { gnutls_log_function2i (level, "(Emacs) " string, extra); } } while (0)
+#define GNUTLS_LOG2i(level, max, string, extra)			\
+  do {								\
+    if ((level) <= (max))					\
+      gnutls_log_function2i (level, "(Emacs) " string, extra);	\
+  } while (false)
 
 extern ptrdiff_t
 emacs_gnutls_write (struct Lisp_Process *proc, const char *buf, ptrdiff_t nbyte);
 extern ptrdiff_t
 emacs_gnutls_read (struct Lisp_Process *proc, char *buf, ptrdiff_t nbyte);
 
-extern int emacs_gnutls_record_check_pending (gnutls_session_t state);
+extern ptrdiff_t emacs_gnutls_record_check_pending (gnutls_session_t state);
+#ifdef WINDOWSNT
 extern void emacs_gnutls_transport_set_errno (gnutls_session_t state, int err);
+#endif
 extern Lisp_Object emacs_gnutls_deinit (Lisp_Object);
 
+#endif
+
 extern void syms_of_gnutls (void);
 
 #endif
-
-#endif
